searchData={"items":[{"type":"module","title":"LlmComposer","doc":"`LlmComposer` is responsible for interacting with a language model to perform chat-related operations,\nsuch as running completions and executing functions based on the responses. The module provides\nfunctionality to handle user messages, generate responses, and automatically execute functions as needed.","ref":"LlmComposer.html"},{"type":"module","title":"Example Usage - LlmComposer","doc":"To use `LlmComposer` for creating a simple chat interaction with a language model, define a settings configuration and initiate a chat:\n\n```elixir\n# Define the settings for your LlmComposer instance\nsettings = %LlmComposer.Settings{\n  model: LlmComposer.Models.OpenAI,\n  model_opts: [model: \"gpt-4o-mini\"],\n  system_prompt: \"You are a helpful assistant.\",\n  user_prompt_prefix: \"\",\n  auto_exec_functions: false,\n  functions: [],\n  api_key: \"\"\n}\n\n# Initiate a simple chat interaction with the defined settings\n{:ok, response} = LlmComposer.simple_chat(settings, \"Hello, how are you?\")\n\n# Print the main response from the assistant\nIO.inspect(response.main_response)\n```","ref":"LlmComposer.html#module-example-usage"},{"type":"module","title":"Output Example - LlmComposer","doc":"Running this code might produce the following log and output:\n\n```\n16:41:07.594 [debug] input_tokens=18, output_tokens=9\n%LlmComposer.Message{\n  type: :assistant,\n  content: \"Hello! How can I assist you today?\"\n}\n```\n\nIn this example, the simple_chat/2 function sends the user's message to the language model using the provided settings, and the response is displayed as the assistant's reply.","ref":"LlmComposer.html#module-output-example"},{"type":"function","title":"LlmComposer.run_completion/3","doc":"Runs the completion process by sending messages to the language model and handling the response.","ref":"LlmComposer.html#run_completion/3"},{"type":"function","title":"Parameters - LlmComposer.run_completion/3","doc":"- `settings`: The settings for the language model, including prompts, model options, and functions.\n  - `messages`: The list of messages to be sent to the language model.\n  - `previous_response` (optional): The previous response object, if any, used for context.","ref":"LlmComposer.html#run_completion/3-parameters"},{"type":"function","title":"Returns - LlmComposer.run_completion/3","doc":"- A tuple containing `:ok` with the response or `:error` if the model call fails.","ref":"LlmComposer.html#run_completion/3-returns"},{"type":"function","title":"LlmComposer.simple_chat/2","doc":"Initiates a simple chat interaction with the language model.","ref":"LlmComposer.html#simple_chat/2"},{"type":"function","title":"Parameters - LlmComposer.simple_chat/2","doc":"- `settings`: The settings for the language model, including prompts and options.\n  - `msg`: The user message to be sent to the language model.","ref":"LlmComposer.html#simple_chat/2-parameters"},{"type":"function","title":"Returns - LlmComposer.simple_chat/2","doc":"- The result of the language model's response, which may include function executions if specified.","ref":"LlmComposer.html#simple_chat/2-returns"},{"type":"type","title":"LlmComposer.messages/0","doc":"","ref":"LlmComposer.html#t:messages/0"},{"type":"module","title":"LlmComposer.Errors","doc":"Custom errors in here","ref":"LlmComposer.Errors.html"},{"type":"module","title":"LlmComposer.Function","doc":"Defines a struct for representing a callable function within the context of a language model interaction.\n\nThis struct is used to define functions that can be executed by the language model based on user prompts or model responses. Each function includes information about its module and function name (`mf`), a unique name for identification, a description of its purpose, and a schema that defines the expected input format.","ref":"LlmComposer.Function.html"},{"type":"module","title":"Fields - LlmComposer.Function","doc":"- `mf`: A tuple `{module, function_name}` specifying the module and the function to be called. The function must accept the arguments defined in the `schema`.\n  - `name`: A string that uniquely identifies the function within the language model's context.\n  - `description`: A brief description of the function, outlining its purpose and expected usage.\n  - `schema`: A map that defines the input schema of the function, detailing the expected parameters and their types. This helps the language model understand how to correctly format the input when calling the function.","ref":"LlmComposer.Function.html#module-fields"},{"type":"module","title":"Example - LlmComposer.Function","doc":"Here's how you might define a function struct for a simple calculator function and implement the corresponding module and function:\n\n```elixir\ndefmodule MyModule do\n  def calculate(%{\"expression\" => expression}) do\n    try do\n      {result, _binding} = Code.eval_string(expression)\n      result\n    rescue\n      _ -> {:error, \"Invalid expression\"}\n    end\n  end\nend\n\n# Define the function struct using the defined module and function\n%LlmComposer.Function{\n  mf: {MyModule, :calculate},\n  name: \"calculator\",\n  description: \"A simple calculator function that evaluates basic math expressions.\",\n  schema: %{\n    type: \"object\",\n    properties: %{\n      expression: %{\n        type: \"string\",\n        description: \"A mathematical expression to evaluate, e.g., '1 + 2'.\"\n      }\n    },\n    required: [\"expression\"]\n  }\n}\n```","ref":"LlmComposer.Function.html#module-example"},{"type":"type","title":"LlmComposer.Function.t/0","doc":"","ref":"LlmComposer.Function.html#t:t/0"},{"type":"module","title":"LlmComposer.FunctionCall","doc":"Helper struct for function call actions.","ref":"LlmComposer.FunctionCall.html"},{"type":"type","title":"LlmComposer.FunctionCall.t/0","doc":"","ref":"LlmComposer.FunctionCall.html#t:t/0"},{"type":"module","title":"LlmComposer.Helpers","doc":"Provides helper functions for the `LlmComposer` module, particularly for managing\nfunction calls and handling language model responses.\n\nThese helpers are designed to execute functions as part of the response processing pipeline,\nmanage completions, and log relevant information for debugging.","ref":"LlmComposer.Helpers.html"},{"type":"function","title":"LlmComposer.Helpers.maybe_complete_chat/3","doc":"Completes the chat flow by appending function results to the messages and re-running the completion process.","ref":"LlmComposer.Helpers.html#maybe_complete_chat/3"},{"type":"function","title":"Parameters - LlmComposer.Helpers.maybe_complete_chat/3","doc":"- `action_result`: The result of a previous action or completion.\n  - `messages`: The list of messages exchanged so far.\n  - `run_completion_fn`: A function to re-run the completion with updated messages.","ref":"LlmComposer.Helpers.html#maybe_complete_chat/3-parameters"},{"type":"function","title":"Returns - LlmComposer.Helpers.maybe_complete_chat/3","doc":"- The result of re-running the completion with the new set of messages and function results.","ref":"LlmComposer.Helpers.html#maybe_complete_chat/3-returns"},{"type":"function","title":"LlmComposer.Helpers.maybe_exec_functions/2","doc":"Executes the functions specified in the language model response, if any.","ref":"LlmComposer.Helpers.html#maybe_exec_functions/2"},{"type":"function","title":"Parameters - LlmComposer.Helpers.maybe_exec_functions/2","doc":"- `res`: The language model response containing actions to be executed.\n  - `llm_functions`: A list of functions available for execution.","ref":"LlmComposer.Helpers.html#maybe_exec_functions/2-parameters"},{"type":"function","title":"Returns - LlmComposer.Helpers.maybe_exec_functions/2","doc":"- `{:ok, res}` if no actions are found in the response.\n  - `{:completion, res, results}` if actions are executed, returning the completion status and results.","ref":"LlmComposer.Helpers.html#maybe_exec_functions/2-returns"},{"type":"type","title":"LlmComposer.Helpers.action_result/0","doc":"","ref":"LlmComposer.Helpers.html#t:action_result/0"},{"type":"type","title":"LlmComposer.Helpers.llmfunctions/0","doc":"","ref":"LlmComposer.Helpers.html#t:llmfunctions/0"},{"type":"type","title":"LlmComposer.Helpers.messages/0","doc":"","ref":"LlmComposer.Helpers.html#t:messages/0"},{"type":"module","title":"LlmComposer.LlmResponse","doc":"Module to parse and easily handle llm responses.","ref":"LlmComposer.LlmResponse.html"},{"type":"function","title":"LlmComposer.LlmResponse.new/2","doc":"","ref":"LlmComposer.LlmResponse.html#new/2"},{"type":"type","title":"LlmComposer.LlmResponse.model_response/0","doc":"","ref":"LlmComposer.LlmResponse.html#t:model_response/0"},{"type":"type","title":"LlmComposer.LlmResponse.t/0","doc":"","ref":"LlmComposer.LlmResponse.html#t:t/0"},{"type":"module","title":"LlmComposer.Message","doc":"Module that represents an arbitrary message for any LLM.","ref":"LlmComposer.Message.html"},{"type":"function","title":"LlmComposer.Message.new/3","doc":"Creates a new message struct with a given type and content.","ref":"LlmComposer.Message.html#new/3"},{"type":"type","title":"LlmComposer.Message.t/0","doc":"","ref":"LlmComposer.Message.html#t:t/0"},{"type":"behaviour","title":"LlmComposer.Model","doc":"Behaviour definition for LLM models.","ref":"LlmComposer.Model.html"},{"type":"callback","title":"LlmComposer.Model.model_id/0","doc":"","ref":"LlmComposer.Model.html#c:model_id/0"},{"type":"callback","title":"LlmComposer.Model.run/3","doc":"","ref":"LlmComposer.Model.html#c:run/3"},{"type":"module","title":"LlmComposer.Models.Bedrock","doc":"Model implementation for Amazon Bedrock.\n\nHandles chat completion requests through Amazon Bedrock Converse API. Any\nBedrock compatible model can be used. To specify any model-specific options for\nthe request, you can pass them in the `request_params` option and they will\nbe merged into the base request that is prepared.","ref":"LlmComposer.Models.Bedrock.html"},{"type":"function","title":"LlmComposer.Models.Bedrock.run/3","doc":"Reference: https://docs.aws.amazon.com/bedrock/latest/APIReference/API_runtime_Converse.html","ref":"LlmComposer.Models.Bedrock.html#run/3"},{"type":"module","title":"LlmComposer.Models.Ollama","doc":"Model implementation for Ollama\n\nBasically it calls the Ollama server api for getting the chat responses.","ref":"LlmComposer.Models.Ollama.html"},{"type":"function","title":"LlmComposer.Models.Ollama.delete/3","doc":"Perform a DELETE request.\n\nSee `request/1` or `request/2` for options definition.\n\n    delete(\"/users\")\n    delete(\"/users\", query: [scope: \"admin\"])\n    delete(client, \"/users\")\n    delete(client, \"/users\", query: [scope: \"admin\"])\n    delete(client, \"/users\", body: %{name: \"Jon\"})","ref":"LlmComposer.Models.Ollama.html#delete/3"},{"type":"function","title":"LlmComposer.Models.Ollama.delete!/3","doc":"Perform a DELETE request.\n\nSee `request!/1` or `request!/2` for options definition.\n\n    delete!(\"/users\")\n    delete!(\"/users\", query: [scope: \"admin\"])\n    delete!(client, \"/users\")\n    delete!(client, \"/users\", query: [scope: \"admin\"])\n    delete!(client, \"/users\", body: %{name: \"Jon\"})","ref":"LlmComposer.Models.Ollama.html#delete!/3"},{"type":"function","title":"LlmComposer.Models.Ollama.get/3","doc":"Perform a GET request.\n\nSee `request/1` or `request/2` for options definition.\n\n    get(\"/users\")\n    get(\"/users\", query: [scope: \"admin\"])\n    get(client, \"/users\")\n    get(client, \"/users\", query: [scope: \"admin\"])\n    get(client, \"/users\", body: %{name: \"Jon\"})","ref":"LlmComposer.Models.Ollama.html#get/3"},{"type":"function","title":"LlmComposer.Models.Ollama.get!/3","doc":"Perform a GET request.\n\nSee `request!/1` or `request!/2` for options definition.\n\n    get!(\"/users\")\n    get!(\"/users\", query: [scope: \"admin\"])\n    get!(client, \"/users\")\n    get!(client, \"/users\", query: [scope: \"admin\"])\n    get!(client, \"/users\", body: %{name: \"Jon\"})","ref":"LlmComposer.Models.Ollama.html#get!/3"},{"type":"function","title":"LlmComposer.Models.Ollama.head/3","doc":"Perform a HEAD request.\n\nSee `request/1` or `request/2` for options definition.\n\n    head(\"/users\")\n    head(\"/users\", query: [scope: \"admin\"])\n    head(client, \"/users\")\n    head(client, \"/users\", query: [scope: \"admin\"])\n    head(client, \"/users\", body: %{name: \"Jon\"})","ref":"LlmComposer.Models.Ollama.html#head/3"},{"type":"function","title":"LlmComposer.Models.Ollama.head!/3","doc":"Perform a HEAD request.\n\nSee `request!/1` or `request!/2` for options definition.\n\n    head!(\"/users\")\n    head!(\"/users\", query: [scope: \"admin\"])\n    head!(client, \"/users\")\n    head!(client, \"/users\", query: [scope: \"admin\"])\n    head!(client, \"/users\", body: %{name: \"Jon\"})","ref":"LlmComposer.Models.Ollama.html#head!/3"},{"type":"function","title":"LlmComposer.Models.Ollama.options/3","doc":"Perform a OPTIONS request.\n\nSee `request/1` or `request/2` for options definition.\n\n    options(\"/users\")\n    options(\"/users\", query: [scope: \"admin\"])\n    options(client, \"/users\")\n    options(client, \"/users\", query: [scope: \"admin\"])\n    options(client, \"/users\", body: %{name: \"Jon\"})","ref":"LlmComposer.Models.Ollama.html#options/3"},{"type":"function","title":"LlmComposer.Models.Ollama.options!/3","doc":"Perform a OPTIONS request.\n\nSee `request!/1` or `request!/2` for options definition.\n\n    options!(\"/users\")\n    options!(\"/users\", query: [scope: \"admin\"])\n    options!(client, \"/users\")\n    options!(client, \"/users\", query: [scope: \"admin\"])\n    options!(client, \"/users\", body: %{name: \"Jon\"})","ref":"LlmComposer.Models.Ollama.html#options!/3"},{"type":"function","title":"LlmComposer.Models.Ollama.patch/4","doc":"Perform a PATCH request.\n\nSee `request/1` or `request/2` for options definition.\n\n    patch(\"/users\", %{name: \"Jon\"})\n    patch(\"/users\", %{name: \"Jon\"}, query: [scope: \"admin\"])\n    patch(client, \"/users\", %{name: \"Jon\"})\n    patch(client, \"/users\", %{name: \"Jon\"}, query: [scope: \"admin\"])","ref":"LlmComposer.Models.Ollama.html#patch/4"},{"type":"function","title":"LlmComposer.Models.Ollama.patch!/4","doc":"Perform a PATCH request.\n\nSee `request!/1` or `request!/2` for options definition.\n\n    patch!(\"/users\", %{name: \"Jon\"})\n    patch!(\"/users\", %{name: \"Jon\"}, query: [scope: \"admin\"])\n    patch!(client, \"/users\", %{name: \"Jon\"})\n    patch!(client, \"/users\", %{name: \"Jon\"}, query: [scope: \"admin\"])","ref":"LlmComposer.Models.Ollama.html#patch!/4"},{"type":"function","title":"LlmComposer.Models.Ollama.post/4","doc":"Perform a POST request.\n\nSee `request/1` or `request/2` for options definition.\n\n    post(\"/users\", %{name: \"Jon\"})\n    post(\"/users\", %{name: \"Jon\"}, query: [scope: \"admin\"])\n    post(client, \"/users\", %{name: \"Jon\"})\n    post(client, \"/users\", %{name: \"Jon\"}, query: [scope: \"admin\"])","ref":"LlmComposer.Models.Ollama.html#post/4"},{"type":"function","title":"LlmComposer.Models.Ollama.post!/4","doc":"Perform a POST request.\n\nSee `request!/1` or `request!/2` for options definition.\n\n    post!(\"/users\", %{name: \"Jon\"})\n    post!(\"/users\", %{name: \"Jon\"}, query: [scope: \"admin\"])\n    post!(client, \"/users\", %{name: \"Jon\"})\n    post!(client, \"/users\", %{name: \"Jon\"}, query: [scope: \"admin\"])","ref":"LlmComposer.Models.Ollama.html#post!/4"},{"type":"function","title":"LlmComposer.Models.Ollama.put/4","doc":"Perform a PUT request.\n\nSee `request/1` or `request/2` for options definition.\n\n    put(\"/users\", %{name: \"Jon\"})\n    put(\"/users\", %{name: \"Jon\"}, query: [scope: \"admin\"])\n    put(client, \"/users\", %{name: \"Jon\"})\n    put(client, \"/users\", %{name: \"Jon\"}, query: [scope: \"admin\"])","ref":"LlmComposer.Models.Ollama.html#put/4"},{"type":"function","title":"LlmComposer.Models.Ollama.put!/4","doc":"Perform a PUT request.\n\nSee `request!/1` or `request!/2` for options definition.\n\n    put!(\"/users\", %{name: \"Jon\"})\n    put!(\"/users\", %{name: \"Jon\"}, query: [scope: \"admin\"])\n    put!(client, \"/users\", %{name: \"Jon\"})\n    put!(client, \"/users\", %{name: \"Jon\"}, query: [scope: \"admin\"])","ref":"LlmComposer.Models.Ollama.html#put!/4"},{"type":"function","title":"LlmComposer.Models.Ollama.request/2","doc":"Perform a request.","ref":"LlmComposer.Models.Ollama.html#request/2"},{"type":"function","title":"Options - LlmComposer.Models.Ollama.request/2","doc":"- `:method` - the request method, one of [`:head`, `:get`, `:delete`, `:trace`, `:options`, `:post`, `:put`, `:patch`]\n- `:url` - either full url e.g. \"http://example.com/some/path\" or just \"/some/path\" if using `Tesla.Middleware.BaseUrl`\n- `:query` - a keyword list of query params, e.g. `[page: 1, per_page: 100]`\n- `:headers` - a keyworld list of headers, e.g. `[{\"content-type\", \"text/plain\"}]`\n- `:body` - depends on used middleware:\n    - by default it can be a binary\n    - if using e.g. JSON encoding middleware it can be a nested map\n    - if adapter supports it it can be a Stream with any of the above\n- `:opts` - custom, per-request middleware or adapter options","ref":"LlmComposer.Models.Ollama.html#request/2-options"},{"type":"function","title":"Examples - LlmComposer.Models.Ollama.request/2","doc":"ExampleApi.request(method: :get, url: \"/users/path\")\n\n    # use shortcut methods\n    ExampleApi.get(\"/users/1\")\n    ExampleApi.post(client, \"/users\", %{name: \"Jon\"})","ref":"LlmComposer.Models.Ollama.html#request/2-examples"},{"type":"function","title":"LlmComposer.Models.Ollama.request!/2","doc":"Perform request and raise in case of error.\n\nThis is similar to `request/2` behaviour from Tesla 0.x\n\nSee `request/2` for list of available options.","ref":"LlmComposer.Models.Ollama.html#request!/2"},{"type":"function","title":"LlmComposer.Models.Ollama.run/3","doc":"Reference: https://github.com/ollama/ollama/blob/main/docs/api.md#generate-a-chat-completion","ref":"LlmComposer.Models.Ollama.html#run/3"},{"type":"function","title":"LlmComposer.Models.Ollama.trace/3","doc":"Perform a TRACE request.\n\nSee `request/1` or `request/2` for options definition.\n\n    trace(\"/users\")\n    trace(\"/users\", query: [scope: \"admin\"])\n    trace(client, \"/users\")\n    trace(client, \"/users\", query: [scope: \"admin\"])\n    trace(client, \"/users\", body: %{name: \"Jon\"})","ref":"LlmComposer.Models.Ollama.html#trace/3"},{"type":"function","title":"LlmComposer.Models.Ollama.trace!/3","doc":"Perform a TRACE request.\n\nSee `request!/1` or `request!/2` for options definition.\n\n    trace!(\"/users\")\n    trace!(\"/users\", query: [scope: \"admin\"])\n    trace!(client, \"/users\")\n    trace!(client, \"/users\", query: [scope: \"admin\"])\n    trace!(client, \"/users\", body: %{name: \"Jon\"})","ref":"LlmComposer.Models.Ollama.html#trace!/3"},{"type":"type","title":"LlmComposer.Models.Ollama.option/0","doc":"Options that may be passed to a request function. See `request/2` for detailed descriptions.","ref":"LlmComposer.Models.Ollama.html#t:option/0"},{"type":"module","title":"LlmComposer.Models.OpenAI","doc":"Model implementation for OpenAI\n\nBasically it calls the OpenAI api for getting the chat responses.","ref":"LlmComposer.Models.OpenAI.html"},{"type":"function","title":"LlmComposer.Models.OpenAI.delete/3","doc":"Perform a DELETE request.\n\nSee `request/1` or `request/2` for options definition.\n\n    delete(\"/users\")\n    delete(\"/users\", query: [scope: \"admin\"])\n    delete(client, \"/users\")\n    delete(client, \"/users\", query: [scope: \"admin\"])\n    delete(client, \"/users\", body: %{name: \"Jon\"})","ref":"LlmComposer.Models.OpenAI.html#delete/3"},{"type":"function","title":"LlmComposer.Models.OpenAI.delete!/3","doc":"Perform a DELETE request.\n\nSee `request!/1` or `request!/2` for options definition.\n\n    delete!(\"/users\")\n    delete!(\"/users\", query: [scope: \"admin\"])\n    delete!(client, \"/users\")\n    delete!(client, \"/users\", query: [scope: \"admin\"])\n    delete!(client, \"/users\", body: %{name: \"Jon\"})","ref":"LlmComposer.Models.OpenAI.html#delete!/3"},{"type":"function","title":"LlmComposer.Models.OpenAI.get/3","doc":"Perform a GET request.\n\nSee `request/1` or `request/2` for options definition.\n\n    get(\"/users\")\n    get(\"/users\", query: [scope: \"admin\"])\n    get(client, \"/users\")\n    get(client, \"/users\", query: [scope: \"admin\"])\n    get(client, \"/users\", body: %{name: \"Jon\"})","ref":"LlmComposer.Models.OpenAI.html#get/3"},{"type":"function","title":"LlmComposer.Models.OpenAI.get!/3","doc":"Perform a GET request.\n\nSee `request!/1` or `request!/2` for options definition.\n\n    get!(\"/users\")\n    get!(\"/users\", query: [scope: \"admin\"])\n    get!(client, \"/users\")\n    get!(client, \"/users\", query: [scope: \"admin\"])\n    get!(client, \"/users\", body: %{name: \"Jon\"})","ref":"LlmComposer.Models.OpenAI.html#get!/3"},{"type":"function","title":"LlmComposer.Models.OpenAI.head/3","doc":"Perform a HEAD request.\n\nSee `request/1` or `request/2` for options definition.\n\n    head(\"/users\")\n    head(\"/users\", query: [scope: \"admin\"])\n    head(client, \"/users\")\n    head(client, \"/users\", query: [scope: \"admin\"])\n    head(client, \"/users\", body: %{name: \"Jon\"})","ref":"LlmComposer.Models.OpenAI.html#head/3"},{"type":"function","title":"LlmComposer.Models.OpenAI.head!/3","doc":"Perform a HEAD request.\n\nSee `request!/1` or `request!/2` for options definition.\n\n    head!(\"/users\")\n    head!(\"/users\", query: [scope: \"admin\"])\n    head!(client, \"/users\")\n    head!(client, \"/users\", query: [scope: \"admin\"])\n    head!(client, \"/users\", body: %{name: \"Jon\"})","ref":"LlmComposer.Models.OpenAI.html#head!/3"},{"type":"function","title":"LlmComposer.Models.OpenAI.options/3","doc":"Perform a OPTIONS request.\n\nSee `request/1` or `request/2` for options definition.\n\n    options(\"/users\")\n    options(\"/users\", query: [scope: \"admin\"])\n    options(client, \"/users\")\n    options(client, \"/users\", query: [scope: \"admin\"])\n    options(client, \"/users\", body: %{name: \"Jon\"})","ref":"LlmComposer.Models.OpenAI.html#options/3"},{"type":"function","title":"LlmComposer.Models.OpenAI.options!/3","doc":"Perform a OPTIONS request.\n\nSee `request!/1` or `request!/2` for options definition.\n\n    options!(\"/users\")\n    options!(\"/users\", query: [scope: \"admin\"])\n    options!(client, \"/users\")\n    options!(client, \"/users\", query: [scope: \"admin\"])\n    options!(client, \"/users\", body: %{name: \"Jon\"})","ref":"LlmComposer.Models.OpenAI.html#options!/3"},{"type":"function","title":"LlmComposer.Models.OpenAI.patch/4","doc":"Perform a PATCH request.\n\nSee `request/1` or `request/2` for options definition.\n\n    patch(\"/users\", %{name: \"Jon\"})\n    patch(\"/users\", %{name: \"Jon\"}, query: [scope: \"admin\"])\n    patch(client, \"/users\", %{name: \"Jon\"})\n    patch(client, \"/users\", %{name: \"Jon\"}, query: [scope: \"admin\"])","ref":"LlmComposer.Models.OpenAI.html#patch/4"},{"type":"function","title":"LlmComposer.Models.OpenAI.patch!/4","doc":"Perform a PATCH request.\n\nSee `request!/1` or `request!/2` for options definition.\n\n    patch!(\"/users\", %{name: \"Jon\"})\n    patch!(\"/users\", %{name: \"Jon\"}, query: [scope: \"admin\"])\n    patch!(client, \"/users\", %{name: \"Jon\"})\n    patch!(client, \"/users\", %{name: \"Jon\"}, query: [scope: \"admin\"])","ref":"LlmComposer.Models.OpenAI.html#patch!/4"},{"type":"function","title":"LlmComposer.Models.OpenAI.post/4","doc":"Perform a POST request.\n\nSee `request/1` or `request/2` for options definition.\n\n    post(\"/users\", %{name: \"Jon\"})\n    post(\"/users\", %{name: \"Jon\"}, query: [scope: \"admin\"])\n    post(client, \"/users\", %{name: \"Jon\"})\n    post(client, \"/users\", %{name: \"Jon\"}, query: [scope: \"admin\"])","ref":"LlmComposer.Models.OpenAI.html#post/4"},{"type":"function","title":"LlmComposer.Models.OpenAI.post!/4","doc":"Perform a POST request.\n\nSee `request!/1` or `request!/2` for options definition.\n\n    post!(\"/users\", %{name: \"Jon\"})\n    post!(\"/users\", %{name: \"Jon\"}, query: [scope: \"admin\"])\n    post!(client, \"/users\", %{name: \"Jon\"})\n    post!(client, \"/users\", %{name: \"Jon\"}, query: [scope: \"admin\"])","ref":"LlmComposer.Models.OpenAI.html#post!/4"},{"type":"function","title":"LlmComposer.Models.OpenAI.put/4","doc":"Perform a PUT request.\n\nSee `request/1` or `request/2` for options definition.\n\n    put(\"/users\", %{name: \"Jon\"})\n    put(\"/users\", %{name: \"Jon\"}, query: [scope: \"admin\"])\n    put(client, \"/users\", %{name: \"Jon\"})\n    put(client, \"/users\", %{name: \"Jon\"}, query: [scope: \"admin\"])","ref":"LlmComposer.Models.OpenAI.html#put/4"},{"type":"function","title":"LlmComposer.Models.OpenAI.put!/4","doc":"Perform a PUT request.\n\nSee `request!/1` or `request!/2` for options definition.\n\n    put!(\"/users\", %{name: \"Jon\"})\n    put!(\"/users\", %{name: \"Jon\"}, query: [scope: \"admin\"])\n    put!(client, \"/users\", %{name: \"Jon\"})\n    put!(client, \"/users\", %{name: \"Jon\"}, query: [scope: \"admin\"])","ref":"LlmComposer.Models.OpenAI.html#put!/4"},{"type":"function","title":"LlmComposer.Models.OpenAI.request/2","doc":"Perform a request.","ref":"LlmComposer.Models.OpenAI.html#request/2"},{"type":"function","title":"Options - LlmComposer.Models.OpenAI.request/2","doc":"- `:method` - the request method, one of [`:head`, `:get`, `:delete`, `:trace`, `:options`, `:post`, `:put`, `:patch`]\n- `:url` - either full url e.g. \"http://example.com/some/path\" or just \"/some/path\" if using `Tesla.Middleware.BaseUrl`\n- `:query` - a keyword list of query params, e.g. `[page: 1, per_page: 100]`\n- `:headers` - a keyworld list of headers, e.g. `[{\"content-type\", \"text/plain\"}]`\n- `:body` - depends on used middleware:\n    - by default it can be a binary\n    - if using e.g. JSON encoding middleware it can be a nested map\n    - if adapter supports it it can be a Stream with any of the above\n- `:opts` - custom, per-request middleware or adapter options","ref":"LlmComposer.Models.OpenAI.html#request/2-options"},{"type":"function","title":"Examples - LlmComposer.Models.OpenAI.request/2","doc":"ExampleApi.request(method: :get, url: \"/users/path\")\n\n    # use shortcut methods\n    ExampleApi.get(\"/users/1\")\n    ExampleApi.post(client, \"/users\", %{name: \"Jon\"})","ref":"LlmComposer.Models.OpenAI.html#request/2-examples"},{"type":"function","title":"LlmComposer.Models.OpenAI.request!/2","doc":"Perform request and raise in case of error.\n\nThis is similar to `request/2` behaviour from Tesla 0.x\n\nSee `request/2` for list of available options.","ref":"LlmComposer.Models.OpenAI.html#request!/2"},{"type":"function","title":"LlmComposer.Models.OpenAI.run/3","doc":"Reference: https://platform.openai.com/docs/api-reference/chat/create","ref":"LlmComposer.Models.OpenAI.html#run/3"},{"type":"function","title":"LlmComposer.Models.OpenAI.trace/3","doc":"Perform a TRACE request.\n\nSee `request/1` or `request/2` for options definition.\n\n    trace(\"/users\")\n    trace(\"/users\", query: [scope: \"admin\"])\n    trace(client, \"/users\")\n    trace(client, \"/users\", query: [scope: \"admin\"])\n    trace(client, \"/users\", body: %{name: \"Jon\"})","ref":"LlmComposer.Models.OpenAI.html#trace/3"},{"type":"function","title":"LlmComposer.Models.OpenAI.trace!/3","doc":"Perform a TRACE request.\n\nSee `request!/1` or `request!/2` for options definition.\n\n    trace!(\"/users\")\n    trace!(\"/users\", query: [scope: \"admin\"])\n    trace!(client, \"/users\")\n    trace!(client, \"/users\", query: [scope: \"admin\"])\n    trace!(client, \"/users\", body: %{name: \"Jon\"})","ref":"LlmComposer.Models.OpenAI.html#trace!/3"},{"type":"type","title":"LlmComposer.Models.OpenAI.option/0","doc":"Options that may be passed to a request function. See `request/2` for detailed descriptions.","ref":"LlmComposer.Models.OpenAI.html#t:option/0"},{"type":"module","title":"LlmComposer.Models.OpenRouter","doc":"Model implementation for OpenRouter\n\nOpenRouter API is very similar to Open AI API, but with some extras like model fallback.","ref":"LlmComposer.Models.OpenRouter.html"},{"type":"function","title":"LlmComposer.Models.OpenRouter.delete/3","doc":"Perform a DELETE request.\n\nSee `request/1` or `request/2` for options definition.\n\n    delete(\"/users\")\n    delete(\"/users\", query: [scope: \"admin\"])\n    delete(client, \"/users\")\n    delete(client, \"/users\", query: [scope: \"admin\"])\n    delete(client, \"/users\", body: %{name: \"Jon\"})","ref":"LlmComposer.Models.OpenRouter.html#delete/3"},{"type":"function","title":"LlmComposer.Models.OpenRouter.delete!/3","doc":"Perform a DELETE request.\n\nSee `request!/1` or `request!/2` for options definition.\n\n    delete!(\"/users\")\n    delete!(\"/users\", query: [scope: \"admin\"])\n    delete!(client, \"/users\")\n    delete!(client, \"/users\", query: [scope: \"admin\"])\n    delete!(client, \"/users\", body: %{name: \"Jon\"})","ref":"LlmComposer.Models.OpenRouter.html#delete!/3"},{"type":"function","title":"LlmComposer.Models.OpenRouter.get/3","doc":"Perform a GET request.\n\nSee `request/1` or `request/2` for options definition.\n\n    get(\"/users\")\n    get(\"/users\", query: [scope: \"admin\"])\n    get(client, \"/users\")\n    get(client, \"/users\", query: [scope: \"admin\"])\n    get(client, \"/users\", body: %{name: \"Jon\"})","ref":"LlmComposer.Models.OpenRouter.html#get/3"},{"type":"function","title":"LlmComposer.Models.OpenRouter.get!/3","doc":"Perform a GET request.\n\nSee `request!/1` or `request!/2` for options definition.\n\n    get!(\"/users\")\n    get!(\"/users\", query: [scope: \"admin\"])\n    get!(client, \"/users\")\n    get!(client, \"/users\", query: [scope: \"admin\"])\n    get!(client, \"/users\", body: %{name: \"Jon\"})","ref":"LlmComposer.Models.OpenRouter.html#get!/3"},{"type":"function","title":"LlmComposer.Models.OpenRouter.head/3","doc":"Perform a HEAD request.\n\nSee `request/1` or `request/2` for options definition.\n\n    head(\"/users\")\n    head(\"/users\", query: [scope: \"admin\"])\n    head(client, \"/users\")\n    head(client, \"/users\", query: [scope: \"admin\"])\n    head(client, \"/users\", body: %{name: \"Jon\"})","ref":"LlmComposer.Models.OpenRouter.html#head/3"},{"type":"function","title":"LlmComposer.Models.OpenRouter.head!/3","doc":"Perform a HEAD request.\n\nSee `request!/1` or `request!/2` for options definition.\n\n    head!(\"/users\")\n    head!(\"/users\", query: [scope: \"admin\"])\n    head!(client, \"/users\")\n    head!(client, \"/users\", query: [scope: \"admin\"])\n    head!(client, \"/users\", body: %{name: \"Jon\"})","ref":"LlmComposer.Models.OpenRouter.html#head!/3"},{"type":"function","title":"LlmComposer.Models.OpenRouter.options/3","doc":"Perform a OPTIONS request.\n\nSee `request/1` or `request/2` for options definition.\n\n    options(\"/users\")\n    options(\"/users\", query: [scope: \"admin\"])\n    options(client, \"/users\")\n    options(client, \"/users\", query: [scope: \"admin\"])\n    options(client, \"/users\", body: %{name: \"Jon\"})","ref":"LlmComposer.Models.OpenRouter.html#options/3"},{"type":"function","title":"LlmComposer.Models.OpenRouter.options!/3","doc":"Perform a OPTIONS request.\n\nSee `request!/1` or `request!/2` for options definition.\n\n    options!(\"/users\")\n    options!(\"/users\", query: [scope: \"admin\"])\n    options!(client, \"/users\")\n    options!(client, \"/users\", query: [scope: \"admin\"])\n    options!(client, \"/users\", body: %{name: \"Jon\"})","ref":"LlmComposer.Models.OpenRouter.html#options!/3"},{"type":"function","title":"LlmComposer.Models.OpenRouter.patch/4","doc":"Perform a PATCH request.\n\nSee `request/1` or `request/2` for options definition.\n\n    patch(\"/users\", %{name: \"Jon\"})\n    patch(\"/users\", %{name: \"Jon\"}, query: [scope: \"admin\"])\n    patch(client, \"/users\", %{name: \"Jon\"})\n    patch(client, \"/users\", %{name: \"Jon\"}, query: [scope: \"admin\"])","ref":"LlmComposer.Models.OpenRouter.html#patch/4"},{"type":"function","title":"LlmComposer.Models.OpenRouter.patch!/4","doc":"Perform a PATCH request.\n\nSee `request!/1` or `request!/2` for options definition.\n\n    patch!(\"/users\", %{name: \"Jon\"})\n    patch!(\"/users\", %{name: \"Jon\"}, query: [scope: \"admin\"])\n    patch!(client, \"/users\", %{name: \"Jon\"})\n    patch!(client, \"/users\", %{name: \"Jon\"}, query: [scope: \"admin\"])","ref":"LlmComposer.Models.OpenRouter.html#patch!/4"},{"type":"function","title":"LlmComposer.Models.OpenRouter.post/4","doc":"Perform a POST request.\n\nSee `request/1` or `request/2` for options definition.\n\n    post(\"/users\", %{name: \"Jon\"})\n    post(\"/users\", %{name: \"Jon\"}, query: [scope: \"admin\"])\n    post(client, \"/users\", %{name: \"Jon\"})\n    post(client, \"/users\", %{name: \"Jon\"}, query: [scope: \"admin\"])","ref":"LlmComposer.Models.OpenRouter.html#post/4"},{"type":"function","title":"LlmComposer.Models.OpenRouter.post!/4","doc":"Perform a POST request.\n\nSee `request!/1` or `request!/2` for options definition.\n\n    post!(\"/users\", %{name: \"Jon\"})\n    post!(\"/users\", %{name: \"Jon\"}, query: [scope: \"admin\"])\n    post!(client, \"/users\", %{name: \"Jon\"})\n    post!(client, \"/users\", %{name: \"Jon\"}, query: [scope: \"admin\"])","ref":"LlmComposer.Models.OpenRouter.html#post!/4"},{"type":"function","title":"LlmComposer.Models.OpenRouter.put/4","doc":"Perform a PUT request.\n\nSee `request/1` or `request/2` for options definition.\n\n    put(\"/users\", %{name: \"Jon\"})\n    put(\"/users\", %{name: \"Jon\"}, query: [scope: \"admin\"])\n    put(client, \"/users\", %{name: \"Jon\"})\n    put(client, \"/users\", %{name: \"Jon\"}, query: [scope: \"admin\"])","ref":"LlmComposer.Models.OpenRouter.html#put/4"},{"type":"function","title":"LlmComposer.Models.OpenRouter.put!/4","doc":"Perform a PUT request.\n\nSee `request!/1` or `request!/2` for options definition.\n\n    put!(\"/users\", %{name: \"Jon\"})\n    put!(\"/users\", %{name: \"Jon\"}, query: [scope: \"admin\"])\n    put!(client, \"/users\", %{name: \"Jon\"})\n    put!(client, \"/users\", %{name: \"Jon\"}, query: [scope: \"admin\"])","ref":"LlmComposer.Models.OpenRouter.html#put!/4"},{"type":"function","title":"LlmComposer.Models.OpenRouter.request/2","doc":"Perform a request.","ref":"LlmComposer.Models.OpenRouter.html#request/2"},{"type":"function","title":"Options - LlmComposer.Models.OpenRouter.request/2","doc":"- `:method` - the request method, one of [`:head`, `:get`, `:delete`, `:trace`, `:options`, `:post`, `:put`, `:patch`]\n- `:url` - either full url e.g. \"http://example.com/some/path\" or just \"/some/path\" if using `Tesla.Middleware.BaseUrl`\n- `:query` - a keyword list of query params, e.g. `[page: 1, per_page: 100]`\n- `:headers` - a keyworld list of headers, e.g. `[{\"content-type\", \"text/plain\"}]`\n- `:body` - depends on used middleware:\n    - by default it can be a binary\n    - if using e.g. JSON encoding middleware it can be a nested map\n    - if adapter supports it it can be a Stream with any of the above\n- `:opts` - custom, per-request middleware or adapter options","ref":"LlmComposer.Models.OpenRouter.html#request/2-options"},{"type":"function","title":"Examples - LlmComposer.Models.OpenRouter.request/2","doc":"ExampleApi.request(method: :get, url: \"/users/path\")\n\n    # use shortcut methods\n    ExampleApi.get(\"/users/1\")\n    ExampleApi.post(client, \"/users\", %{name: \"Jon\"})","ref":"LlmComposer.Models.OpenRouter.html#request/2-examples"},{"type":"function","title":"LlmComposer.Models.OpenRouter.request!/2","doc":"Perform request and raise in case of error.\n\nThis is similar to `request/2` behaviour from Tesla 0.x\n\nSee `request/2` for list of available options.","ref":"LlmComposer.Models.OpenRouter.html#request!/2"},{"type":"function","title":"LlmComposer.Models.OpenRouter.run/3","doc":"Reference: https://openrouter.ai/docs/api-reference/chat-completion","ref":"LlmComposer.Models.OpenRouter.html#run/3"},{"type":"function","title":"LlmComposer.Models.OpenRouter.trace/3","doc":"Perform a TRACE request.\n\nSee `request/1` or `request/2` for options definition.\n\n    trace(\"/users\")\n    trace(\"/users\", query: [scope: \"admin\"])\n    trace(client, \"/users\")\n    trace(client, \"/users\", query: [scope: \"admin\"])\n    trace(client, \"/users\", body: %{name: \"Jon\"})","ref":"LlmComposer.Models.OpenRouter.html#trace/3"},{"type":"function","title":"LlmComposer.Models.OpenRouter.trace!/3","doc":"Perform a TRACE request.\n\nSee `request!/1` or `request!/2` for options definition.\n\n    trace!(\"/users\")\n    trace!(\"/users\", query: [scope: \"admin\"])\n    trace!(client, \"/users\")\n    trace!(client, \"/users\", query: [scope: \"admin\"])\n    trace!(client, \"/users\", body: %{name: \"Jon\"})","ref":"LlmComposer.Models.OpenRouter.html#trace!/3"},{"type":"type","title":"LlmComposer.Models.OpenRouter.option/0","doc":"Options that may be passed to a request function. See `request/2` for detailed descriptions.","ref":"LlmComposer.Models.OpenRouter.html#t:option/0"},{"type":"module","title":"LlmComposer.Settings","doc":"Defines the settings for configuring chat interactions with a language model.\n\nThis module provides a struct that includes model configuration, prompt settings, and options for function execution, enabling fine control over the chat flow and behavior.","ref":"LlmComposer.Settings.html"},{"type":"type","title":"LlmComposer.Settings.t/0","doc":"","ref":"LlmComposer.Settings.html#t:t/0"},{"type":"exception","title":"LlmComposer.Errors.MissingKeyError","doc":"","ref":"LlmComposer.Errors.MissingKeyError.html"},{"type":"extras","title":"LlmComposer","doc":"# LlmComposer\n\n**LlmComposer** is an Elixir library that simplifies the interaction with large language models (LLMs) such as OpenAI's GPT, providing a streamlined way to build and execute LLM-based applications or chatbots. It currently supports multiple model providers, including OpenAI, OpenRouter, Ollama or Bedrock, with features like auto-execution of functions and customizable prompts to cater to different use cases.","ref":"readme.html"},{"type":"extras","title":"Installation - LlmComposer","doc":"If [available in Hex](https://hex.pm/docs/publish), the package can be installed\nby adding `llm_composer` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:llm_composer, \"~> 0.3.0\"}\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Usage - LlmComposer","doc":"","ref":"readme.html#usage"},{"type":"extras","title":"Simple Bot Definition - LlmComposer","doc":"To create a basic chatbot using LlmComposer, you need to define a module that uses the `LlmComposer.Caller` behavior. The example below demonstrates a simple configuration with OpenAI as the model provider:\n\n```elixir\nApplication.put_env(:llm_composer, :openai_key, \" \")\n\ndefmodule MyChat do\n\n  @settings %LlmComposer.Settings{\n    model: LlmComposer.Models.OpenAI,\n    model_opts: [model: \"gpt-4o-mini\"],\n    system_prompt: \"You are a helpful assistant.\"\n  }\n\n  def simple_chat(msg) do\n    LlmComposer.simple_chat(@settings, msg)\n  end\nend\n\n{:ok, res} = MyChat.simple_chat(\"hi\")\n\nIO.inspect(res.main_response)\n```\n\nExample of execution:\n\n```\nmix run sample.ex\n\n16:41:07.594 [debug] input_tokens=18, output_tokens=9\nLlmComposer.Message.new(\n  :assistant,\n  \"Hello! How can I assist you today?\"\n)\n```\n\nThis will trigger a conversation with the assistant based on the provided system prompt.","ref":"readme.html#simple-bot-definition"},{"type":"extras","title":"Using old messages - LlmComposer","doc":"For more control over the interactions, basically to send the messages history and track the context, you can use the `run_completion/3` function directly.\n\nHere’s an example that demonstrates how to use `run_completion` with a custom message flow:\n\n```elixir\nApplication.put_env(:llm_composer, :openai_key, \" \")\n\ndefmodule MyCustomChat do\n\n  @settings %LlmComposer.Settings{\n    model: LlmComposer.Models.OpenAI,\n    model_opts: [model: \"gpt-4o-mini\"],\n    system_prompt: \"You are an assistant specialized in history.\",\n    auto_exec_functions: false,\n    functions: []\n  }\n\n  def run_custom_chat() do\n    # Define a conversation history with user and assistant messages\n    messages = [\n      LlmComposer.Message.new(:user, \"What is the Roman Empire?\"),\n      LlmComposer.Message.new(:assistant, \"The Roman Empire was a period of ancient Roman civilization with an autocratic government.\"),\n      LlmComposer.Message.new(:user, \"When did it begin?\")\n    ]\n\n    {:ok, res} = LlmComposer.run_completion(@settings, messages)\n\n    res.main_response\n  end\nend\n\nIO.inspect(MyCustomChat.run_custom_chat())\n```\n\nExample of execution:\n\n```\nmix run custom_chat.ex\n\n16:45:10.123 [debug] input_tokens=85, output_tokens=47\nLlmComposer.Message.new(\n  :assistant,\n  \"The Roman Empire began in 27 B.C. after the end of the Roman Republic, and it continued until 476 A.D. in the West.\"\n)\n```","ref":"readme.html#using-old-messages"},{"type":"extras","title":"Using Ollama Backend - LlmComposer","doc":"LlmComposer also supports the Ollama backend, allowing interaction with models hosted on Ollama.\n\nMake sure to start the Ollama server first.\n\n```elixir\n# Set the Ollama URI in the application environment if not already configured\n# Application.put_env(:llm_composer, :ollama_uri, \"http://localhost:11434\")\n\ndefmodule MyChat do\n\n  @settings %LlmComposer.Settings{\n    model: LlmComposer.Models.Ollama,\n    model_opts: [model: \"llama3.1\"],\n    system_prompt: \"You are a helpful assistant.\"\n  }\n\n  def simple_chat(msg) do\n    LlmComposer.simple_chat(@settings, msg)\n  end\nend\n\n{:ok, res} = MyChat.simple_chat(\"hi\")\n\nIO.inspect(res.main_response)\n```\n\nExample of execution:\n\n```\nmix run sample_ollama.ex\n\n17:08:34.271 [debug] input_tokens=, output_tokens=\nLlmComposer.Message.new(\n  :assistant,\n  \"How can I assist you today?\",\n  %{\n    original: %{\n      \"content\" => \"How can I assist you today?\",\n      \"role\" => \"assistant\"\n    }\n  }\n)\n```\n\nNo function calls support in Ollama (for now)","ref":"readme.html#using-ollama-backend"},{"type":"extras","title":"Using OpenRouter - LlmComposer","doc":"LlmComposer supports integration with [OpenRouter](https://openrouter.ai/), giving you access to a variety of LLM models through a single API compatible with OpenAI's interface. Also supports, the OpenRouter's feature of setting fallback models.\n\nTo use OpenRouter with LlmComposer, you'll need to:\n\n1. Sign up for an API key from [OpenRouter](https://openrouter.ai/)\n2. Configure your application to use OpenRouter's endpoint\n\nHere's a complete example:\n\n```elixir\n# Configure the OpenRouter API key and endpoint\nApplication.put_env(:llm_composer, :open_router_key, \" \")\n\ndefmodule MyOpenRouterChat do\n  @settings %LlmComposer.Settings{\n    model: LlmComposer.Models.OpenRouter,\n    # Use any model available on OpenRouter\n    model_opts: [model: \"anthropic/claude-3-sonnet\", models: [\"openai/gpt-4o\", \"fallback-model2\"]],\n    system_prompt: \"You are a SAAS consultant\"\n  }\n\n  def simple_chat(msg) do\n    LlmComposer.simple_chat(@settings, msg)\n  end\nend\n\n{:ok, res} = MyOpenRouterChat.simple_chat(\"Why doofinder is so awesome?\")\n\nIO.inspect(res.main_response)\n```\n\nExample of execution:\n\n```\nmix run openrouter_sample.ex\n\n17:12:45.124 [debug] input_tokens=42, output_tokens=156\nLlmComposer.Message.new(\n  :assistant,\n  \"Doofinder is an excellent site search solution for ecommerce websites. Here are some reasons why Doofinder is considered awesome:...\n)\n```","ref":"readme.html#using-openrouter"},{"type":"extras","title":"Using AWS Bedrock - LlmComposer","doc":"LlmComposer also integrates with [Bedrock](https://aws.amazon.com/es/bedrock/) via its Converse API. This allows you tu use Bedrock as provider with any of its supported models.\n\nCurrently, function execution is **not supported** with Bedrock.\n\nTo integrate with Bedrock, LlmComposer uses the [`ex_aws`](https://hexdocs.pm/ex_aws/readme.html#aws-key-configuration) to perform its requests. So, if you plan to use Bedrock, make sure that you have configured `ex_aws` as per the official documentation of the library.\n\nHere's a complete example:\n\n```elixir\n# In your config files:\nconfig :ex_aws,\n  access_key_id: \"your key\",\n  secret_access_key: \"your secret\"\n---\n\ndefmodule MyBedrockChat do\n  @settings %LlmComposer.Settings{\n    model: LlmComposer.Models.Bedrock,\n    # Use any model available Bedrock model\n    model_opts: [model: \"eu.amazon.nova-lite-v1:0\"],\n    system_prompt: \"You are an expert in Quantum Field Theory.\"\n  }\n\n  def simple_chat(msg) do\n    LlmComposer.simple_chat(@settings, msg)\n  end\nend\n\n{:ok, res} = MyBedrockChat.simple_chat(\"What is the wave function collapse? Just a few sentences\")\n\nIO.inspect(res.main_response)\n```\n\nExample of execution:\n\n```\n%LlmComposer.Message{\n  type: :assistant,\n  content: \"Wave function collapse is a concept in quantum mechanics that describes the transition of a quantum system from a superposition of states to a single definite state upon measurement. This phenomenon is often associated with the interpretation of quantum mechanics, particularly the Copenhagen interpretation, and it remains a topic of ongoing debate and research in the field.\"\n}\n```","ref":"readme.html#using-aws-bedrock"},{"type":"extras","title":"Bot with external function call - LlmComposer","doc":"You can enhance the bot's capabilities by adding support for external function execution. This example demonstrates how to add a simple calculator that evaluates basic math expressions:\n\n```elixir\nApplication.put_env(:llm_composer, :openai_key, \" \")\n\ndefmodule MyChat do\n\n  @settings %LlmComposer.Settings{\n    model: LlmComposer.Models.OpenAI,\n    model_opts: [model: \"gpt-4o-mini\"],\n    system_prompt: \"You are a helpful math assistant that assists with calculations.\",\n    auto_exec_functions: true,\n    functions: [\n      %LlmComposer.Function{\n        mf: {__MODULE__, :calculator},\n        name: \"calculator\",\n        description: \"A calculator that accepts math expressions as strings, e.g., '1 * (2 + 3) / 4', supporting the operators ['+', '-', '*', '/'].\",\n        schema: %{\n          type: \"object\",\n          properties: %{\n            expression: %{\n              type: \"string\",\n              description: \"A math expression to evaluate, using '+', '-', '*', '/'.\",\n              example: \"1 * (2 + 3) / 4\"\n            }\n          },\n          required: [\"expression\"]\n        }\n      }\n    ]\n  }\n\n  def simple_chat(msg) do\n    LlmComposer.simple_chat(@settings, msg)\n  end\n\n  @spec calculator(map()) :: number() | {:error, String.t()}\n  def calculator(%{\"expression\" => expression}) do\n    # Basic validation pattern to prevent arbitrary code execution\n    pattern = ~r/^[0-9\\.\\s\\+\\-\\*\\/\\(\\)]+$/\n\n    if Regex.match?(pattern, expression) do\n      try do\n        {result, _binding} = Code.eval_string(expression)\n        result\n      rescue\n        _ -> {:error, \"Invalid expression\"}\n      end\n    else\n      {:error, \"Invalid expression format\"}\n    end\n  end\nend\n\n{:ok, res} = MyChat.simple_chat(\"hi, how much is 1 + 2?\")\n\nIO.inspect(res.main_response)\n```\n\nExample of execution:\n\n```\nmix run functions_sample.ex\n\n16:38:28.338 [debug] input_tokens=111, output_tokens=17\n\n16:38:28.935 [debug] input_tokens=136, output_tokens=9\nLlmComposer.Message.new(\n  :assistant,\n  \"1 + 2 is 3.\"\n)\n```\n\nIn this example, the bot first calls OpenAI to understand the user's intent and determine that a function (the calculator) should be executed. The function is then executed locally, and the result is sent back to the user in a second API call.","ref":"readme.html#bot-with-external-function-call"},{"type":"extras","title":"Additional Features - LlmComposer","doc":"* Auto Function Execution: Automatically executes predefined functions, reducing manual intervention.\n* System Prompts: Customize the assistant's behavior by modifying the system prompt (e.g., creating different personalities or roles for your bot).\n\n---\n\nDocumentation can be generated with [ExDoc](https://github.com/elixir-lang/ex_doc)\nand published on [HexDocs](https://hexdocs.pm). Once published, the docs can\nbe found at  .","ref":"readme.html#additional-features"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,52,46,50]}}